#ifndef __CLEANUP_HEADER_INCLUDED__
#define __CLEANUP_HEADER_INCLUDED__

#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#define EXIT_OK		0
#define EXIT_ERR	1
#define EXIT_DBG	2

/* A cleanup callback handler */
typedef void(*proc_cleanup)(int, void *);

/* Cleanup list element */
struct clean_list {
	struct clean_list 	*next;
	proc_cleanup		cleanup;
	void			*priv;
};

/* Static inline functions */
static inline const char *get_nerr(int num) {
	return (num>=sys_nerr) ?
		"[unknown error]" :
		sys_errlist[num];
}
static inline const char *get_err(void){return get_nerr(errno);}

/* Cleanup API */
void __attribute__((noreturn)) cleanup(int, const char *, ...);
void cleanup_add(proc_cleanup, void *);
void cleanup_final(proc_cleanup, void *);

/* Cleanup helper macros */
#define cperror(x) \
	cleanup(EXIT_ERR, __FUNCTION__ "(): " x "(): %s", get_err());

#endif /* __CLEANUP_HEADER_INCLUDED__ */
