/*
* This file is part of Firestorm NIDS
* Copyright (c) 2002 Gianni Tedesco
*
*    Track relevent ruleset parser modules. Ruleset parsing is actually
*    done inside the plugins.
*/
#include <rlim.h>
#include <string.h>
#include <cleanup.h>
#include <parser.h>

struct parser *parse_list=NULL;
unsigned int num_parsers=0;
unsigned int parser_warn=0;

void parser_cleanup(int code, void *priv)
{
	struct parser *foo, *bar;
	
	printf("Unregistering all parsers\n");

	for(foo=parse_list; foo;) {
		bar=foo;
		foo=foo->next;
		free(bar);
	}
}

void parser_init(void)
{
	cleanup_add(parser_cleanup, NULL);
}

int parser_load(char *type, char *file)
{
	struct parser *p;

	if ( !type ) return 0;

	for(p=parse_list; p; p=p->next) {
		if ( !strcmp(p->name, type) ) {
			return p->load_file(file);
		}
	}
	return 0;
}

int parser_add(const char *name, proc_parser_run r)
{
	struct parser *p;

	if ( !name || !r ) return 0;

	if ( num_parsers >= MAX_PARSERS ) {
		if ( !parser_warn ) {
			parser_warn++;
			printf("parser: warning: MAX_PARSERS exceeded.\n");
		}
		return 0;
	}

	/* TODO: check for duplicates */

	if ( !(p=malloc(sizeof(struct parser))) ) {
		cperror("malloc");
	}

	/* Append to list */
	p->name=name;
	p->load_file=r;
	p->next=parse_list;
	parse_list=p;
	num_parsers++;
	
	return 1;
}
