#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <packet.h>
#include <capture.h>
#include <capdev.h>
#include <decode.h>
#include <target.h>
#include <plugin.h>

int llen=16;

action_t alert(struct packet *, char *, void *);
int alert_v(char *, void **);
void alert_c(void *);

struct target alert_t[]={
	target_init("alert", alert, alert_v, alert_c),
	target_null()
};

void hex_dump(char *ptr, unsigned int len)
{
	int i, j;
	int line;
	unsigned char *tmp=(unsigned char *)ptr;

	for(j=0; j<len; ) {
		if ( j+llen > len ) {
			line=len-j;
		}else{
			line=llen;
		}

		printf("%05X : ", j);

		for(i=0; i<llen; i++) {
			if ( i<line ) {
				if ( isprint(tmp[i]) ) {
					printf("%c", tmp[i]);
				}else{
					printf(".");
				}
			}else{
				printf(" ");
			}
		}
		
		for(i=0; i<line; i++) {
			printf(" %02X", tmp[i]);
		}

		j+=line;
		tmp+=line;
		printf("\n");
	}
}

action_t alert(struct packet *pkt, char *text, void *priv)
{
	int i;
	char ln[128];

	printf("2002-03-03 15:21:32: %s[%s]: CAPLEN=%u: %s\n",
		pkt->capture->capdev->name,
		pkt->capture->interface,
		pkt->caplen,
		(char *)text);
	
	for(i=0; i<pkt->llen; i++) {
		if ( !pkt->layer[i].proto ) {
			printf("    data: Application layer data (%u bytes)\n",
				pkt->end-pkt->layer[i].h.raw);
			continue;
		}

		if ( pkt->layer[i].proto->print ) {
			pkt->layer[i].proto->print(
				&pkt->layer[i], ln, 82);
		}else{
			sprintf(ln, "(no information)");
		}
		printf("%8s: %s\n",
			pkt->layer[i].proto->name,
			ln);
	}

	/* hex dump the mofo */
	hex_dump(pkt->base, pkt->caplen);

	printf("\n");
	fflush(stdout);
	return TARGET_DONE;
}

int alert_v(char *args, void **ptr){*ptr=NULL; return 1;}
void alert_c(void *priv){}

int _PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{
	proc_target_add target_add;
	int ok=0;
	
	plugin_check(in, out);
	
	PLUGIN_ID("target.alert", "Print a message to stdout");
	PLUGIN_VERSION(2, 0);
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@ecsc.co.uk");
	PLUGIN_LICENSE("GPL");
	
	if ( !(target_add=in->import("target.add")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	ok+=target_add(alert_t);
	
	return (ok) ? PLUGIN_ERR_OK : PLUGIN_ERR_FAIL;
}

int _PLUGIN_UNLOAD (int code) {
	return PLUGIN_ERR_OK;
}
